@@ -752,6 +752,16 @@ void SessionController::setupCommonActions()
     action->setIcon(QIcon::fromTheme(QStringLiteral("document-properties")));
     setEditProfileActionText(SessionManager::instance()->sessionProfile(session()));
 
+    action = collection->addAction(QStringLiteral("next-profile"), this, &SessionController::nextProfile);
+    action->setIcon(QIcon::fromTheme(QStringLiteral("document-properties")));
+    action->setText(i18n("Next Profile"));
+    collection->setDefaultShortcut(action, Qt::CTRL | Qt::ALT | Qt::Key_N);
+
+    action = collection->addAction(QStringLiteral("prev-profile"), this, &SessionController::prevProfile);
+    action->setIcon(QIcon::fromTheme(QStringLiteral("document-properties")));
+    action->setText(i18n("Previous Profile"));
+    collection->setDefaultShortcut(action, Qt::CTRL | Qt::ALT | Qt::Key_M);
+
     _switchProfileMenu = new KActionMenu(QIcon::fromTheme(QStringLiteral("exchange-positions")), i18n("Switch Profile"), this);
     collection->addAction(QStringLiteral("switch-profile"), _switchProfileMenu);
     connect(_switchProfileMenu->menu(), &QMenu::aboutToShow, this, &Konsole::SessionController::prepareSwitchProfileMenu);
@@ -799,7 +809,8 @@ void SessionController::setupCommonActions()
     connect(action, &QAction::toggled, this, [this, action]() {
         toggleAllowMouseTracking(action);
     });
-    action->setText(i18nc("@item:inmenu Allows terminal applications to request mouse tracking", "Allow mouse tracking"));
+    action->setText(i18nc("@item:inmenu Allows terminal applications to request mouse tracking", "Allow Mouse Tracking"));
+    action->setIcon(QIcon::fromTheme(QStringLiteral("input-mouse-symbolic")));
     action->setCheckable(true);
 
     // Read-only
@@ -807,7 +818,7 @@ void SessionController::setupCommonActions()
     connect(action, &QAction::toggled, this, [this, action]() {
         toggleReadOnly(action);
     });
-    action->setText(i18nc("@item:inmenu A read only (locked) session", "Read-only"));
+    action->setText(i18nc("@item:inmenu A read only (locked) session", "Read-Only"));
     action->setCheckable(true);
     updateReadOnlyActionStates();
 }
@@ -858,7 +869,7 @@ void SessionController::setupExtraActions()
     collection->setDefaultShortcut(action, Qt::CTRL | Qt::ALT | Qt::Key_U);
 
     // Monitor
-    KToggleAction *toggleAction = new KToggleAction(i18n("One-shot monitors"), this);
+    KToggleAction *toggleAction = new KToggleAction(i18n("One-Shot Monitors"), this);
     action = collection->addAction(QStringLiteral("monitor-once"), toggleAction);
     connect(action, &QAction::toggled, this, &Konsole::SessionController::monitorOnce);
     action->setIcon(QIcon::fromTheme(QStringLiteral("tools-media-optical-burn")));
@@ -1020,6 +1031,20 @@ void SessionController::editCurrentProfile()
     dialog->show();
 }
 
+void SessionController::nextProfile()
+{
+    auto profile = SessionManager::instance()->sessionProfile(session());
+    auto next = *ProfileManager::instance()->nextProfile(profile);
+    SessionManager::instance()->setSessionProfile(session(), next);
+}
+
+void SessionController::prevProfile()
+{
+    auto profile = SessionManager::instance()->sessionProfile(session());
+    auto prev = *ProfileManager::instance()->prevProfile(profile);
+    SessionManager::instance()->setSessionProfile(session(), prev);
+}
+
 void SessionController::renameSession()
 {
     const QString sessionLocalTabTitleFormat = session()->tabTitleFormat(Session::LocalTabTitle);
@@ -1537,6 +1562,14 @@ bool SessionController::reverseSearchChecked() const
     return options.at(IncrementalSearchBar::ReverseSearch);
 }
 
+bool SessionController::noWrapChecked() const
+{
+    Q_ASSERT(_searchBar);
+
+    QBitArray options = _searchBar->optionsChecked();
+    return options.at(IncrementalSearchBar::NoWrap);
+}
+
 QRegularExpression SessionController::regexpFromSearchBarOptions() const
 {
     QBitArray options = _searchBar->optionsChecked();
@@ -1608,7 +1641,7 @@ void SessionController::searchTextChanged(const QString &text)
 
     // update search.  this is called even when the text is
     // empty to clear the view's filters
-    beginSearch(text, reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch);
+    beginSearch(text, reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch, noWrapChecked());
 }
 void SessionController::searchCompleted(bool success)
 {
@@ -1619,7 +1652,7 @@ void SessionController::searchCompleted(bool success)
     }
 }
 
-void SessionController::beginSearch(const QString &text, Enum::SearchDirection direction)
+void SessionController::beginSearch(const QString &text, Enum::SearchDirection direction, bool noWrap)
 {
     Q_ASSERT(_searchBar);
     Q_ASSERT(_searchFilter);
@@ -1643,6 +1676,7 @@ void SessionController::beginSearch(const QString &text, Enum::SearchDirection d
 
         task->setRegExp(regExp);
         task->setSearchDirection(direction);
+        task->setNoWrap(noWrap);
         task->setAutoDelete(true);
         task->setStartLine(_searchStartLine);
         task->addScreenWindow(session(), view()->screenWindow());
@@ -1676,7 +1710,7 @@ void SessionController::searchFrom()
         setSearchStartTo(0);
     }
 
-    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch);
+    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch, noWrapChecked());
 }
 void SessionController::findNextInHistory()
 {
@@ -1685,7 +1719,7 @@ void SessionController::findNextInHistory()
 
     setSearchStartTo(_prevSearchResultLine);
 
-    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch);
+    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch, noWrapChecked());
 }
 void SessionController::findPreviousInHistory()
 {
@@ -1694,7 +1728,7 @@ void SessionController::findPreviousInHistory()
 
     setSearchStartTo(_prevSearchResultLine);
 
-    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::ForwardsSearch : Enum::BackwardsSearch);
+    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::ForwardsSearch : Enum::BackwardsSearch, noWrapChecked());
 }
 void SessionController::updateMenuIconsAccordingToReverseSearchSetting()
 {
@@ -1713,7 +1747,7 @@ void SessionController::changeSearchMatch()
 
     // reset Selection for new case match
     view()->clearMouseSelection();
-    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch);
+    beginSearch(_searchBar->searchText(), reverseSearchChecked() ? Enum::BackwardsSearch : Enum::ForwardsSearch, noWrapChecked());
 }
 void SessionController::showHistoryOptions()
 {
